#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from subprocess import Popen, PIPE
import crypt
import logging
import os
import socket
import subprocess
import traceback

from blivet.size import Size

from __data__ import *
from lib.events import EventDispatcher
from logger import createFailureLog
from logger import createLogger
from model.config import *
from model.installer import Installer
from model.kernelcmdline import KernelCmdLine
from model.licensemodel import getIBMLicense
from model.licensemodel import getNonIBMLicenseNotices
from model.network import reverseDNSLookup
from model.network import searchDNSFromFQDN

from modules.execscripts.execpostscripts import ExecPostScripts
from modules.finalsetup.finalsetup import FinalSetup
from modules.i18n import i18n
from modules.identifysystem.identifysystem import IdentifySystem
from modules.installauto.installauto import InstallAuto
from modules.network.dns import DNS
from modules.network.network import Network
from modules.network.network_interface import NetworkInterface
from modules.network.netsetup import NetSetup
from modules.network.networktopology import NetworkTopology
from modules.network.ntp import NTP
from modules.rootpassword.rootpassword import RootPassword
from modules.timezone.timezone import Timezone

from pkvmerror import PKVMError
from screen_ids import ScreenIds
from storage.autopartitionerlvm import AutoPartitionerLVM




#
# CODE
#
class Controller(object):
    """
    Orchestrates the installer
    """

    def __init__(self, model, viewer):
        """
        Constructs the controller
        """
        self.__model = model
        self.__viewer = viewer
        self.screenControllers = {
            ScreenIds.CONFIRM_INSTALLATION: self.confirmInstallationScreen,
            ScreenIds.SPLASH: self.splashScreen,
            ScreenIds.GENERAL_ERROR: None,
            ScreenIds.INSTALLATION_PROGRESS: self.installationProgressScreen,
            ScreenIds.INSTALLATION_SUMMARY: self.summaryScreen,
            ScreenIds.LICENSE: self.licenseScreen,
            ScreenIds.LIST_NETWORK_INTERFACES: self.listNwInterfacesScreen,
            ScreenIds.MENU: self.menuScreen,
            ScreenIds.MESSAGE: None,
            ScreenIds.REBOOT: self.rebootSystem,
            ScreenIds.REINSTALLATION_ERROR: None,
            ScreenIds.SELECT_DISKS: self.selectDisksScreen,
            ScreenIds.SELECT_LANGUAGE: self.setLanguageScreen,
            ScreenIds.SELECT_PREV_INSTALL: self.selectPrevInstallationScreen,
            ScreenIds.SELECT_TIMEZONE: self.selectTimezoneScreen,
            ScreenIds.SETUP_DATETIME: self.setupDateTimeScreen,
            ScreenIds.SETUP_DNS: self.setupDnsScreen,
            ScreenIds.SETUP_LVM: self.setupLvmScreen,
            ScreenIds.SETUP_NETWORK_INTERFACE: self.setupNwInterfaceScreen,
            ScreenIds.SETUP_NTP: self.setupNtpScreen,
            ScreenIds.SETUP_ROOT_PASSWORD: self.setupRootPasswordScreen
        }

        try:
            self.event_dispatcher = EventDispatcher()

            self.cmdLine = KernelCmdLine()
            self.__model.insert('isManual',
                                not self.cmdLine.hasKey('kvmp.inst.auto'))
            self.__logger = createLogger()
            self.__installer = Installer()
            self.__initialized = False

            self.splashScreen({})

            # set the mount point for the installation
            self.__model.insert('mountDir', INSTALL_ROOT_DIR)

            if self.__model.get('isManual'):
                self.__logger.info('Manual mode')
                firstScreen = ScreenIds.SELECT_LANGUAGE
            else:
                self.__logger.info("Unattended installation (auto mode - kickstart)")
                firstScreen = ScreenIds.INSTALLATION_PROGRESS
                self.initAfterLanguageSet()

                if self.__model.get('preserveData'):
                    self.initStorageDataWithPreservation()
                else:
                    self.initStorageData()

            self.runInstallerUI(firstScreen, {})

        except PKVMError as e:
            createFailureLog()
            viewInput = {"errorMsg": e.getErrorMessage(),
                         "productVersion": STR_VERSION,
                         "errorLogTarFile": ERROR_ARCHIVE_PATH}
            self.__viewer.runView(ScreenIds.GENERAL_ERROR, viewInput)
            self.rebootSystem(True)
        except Exception as e:
            pkvmError = PKVMError("000")
            createFailureLog()
            viewInput = {"errorMsg": pkvmError.getErrorMessage(),
                         "productVersion": STR_VERSION,
                         "errorLogTarFile": ERROR_ARCHIVE_PATH}
            self.__viewer.runView(ScreenIds.GENERAL_ERROR, viewInput)
            self.rebootSystem(True)

    def __saveNwInterfaceConfig(self, nwInterfaceData, nwInterface):
        """
        Write network interface configuration to system

        Args:
            nwInterfaceData (dict): network interface data set in UI
            nwInterface (modules.network_interface.NetworkInterface): network interface object
        """

        # bridge
        if nwInterfaceData["bridge"]:
            nwInterface.setBridge(True)
        else:
            nwInterface.setBridge(False)

        # device
        nwInterface.setDev(nwInterfaceData["name"], nwInterfaceData["mac"])

        # onboot
        nwInterface.setOnBoot(nwInterfaceData["onboot"])

        # dns
        nwInterface.setDNS()

        # dhcp
        nwInterface.enableDHCP(nwInterfaceData["dhcp"])
        if not nwInterfaceData["dhcp"]:
            nwInterface.setIp(nwInterfaceData["ip"])
            nwInterface.setNetmask(nwInterfaceData["netmask"])
            nwInterface.setGateway(nwInterfaceData["gateway"])

        # Saving network cfg file
        Network.writeConfigFile(nwInterface.getEthConfigFile())
        if nwInterface.hasBridge():
            Network.writeConfigFile(nwInterface.getBrConfigFile())
        else:
            Network.deleteBrConfigFile(nwInterfaceData["name"])

    def __registerEventListeners(self):
        """
        Register event listeners for PRE_INSTALL and POST_INSTALL
            events generated on the installer.
        """

        self.event_dispatcher.register("PRE_INSTALL", InstallAuto().onPreInstall)
        self.event_dispatcher.register("PRE_INSTALL", NetworkTopology().onPreInstall)
        self.event_dispatcher.register("PRE_INSTALL", NetworkInterface().onPreInstall)
        self.event_dispatcher.register("PRE_INSTALL", IdentifySystem().onPreInstall)

        self.event_dispatcher.register("POST_INSTALL", RootPassword().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", Timezone().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", NTP().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", NetSetup().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", DNS().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", ExecPostScripts().onPostInstall)
        self.event_dispatcher.register("POST_INSTALL", FinalSetup().onPostInstall)


    def runInstallerUI(self, firstScreen, firstScreenParams):
        """
        Runs the installer UI
        """

        currScreen = firstScreen
        currScreenParams = firstScreenParams

        # screen transition loop
        while True:

            nextScreen, nextScreenParams = self.screenControllers[currScreen](currScreenParams)
            currScreen = nextScreen
            currScreenParams = nextScreenParams

    def getDisksInformation(self):

        viewInput = {"msg": GETTING_DISK_INFORMATION.localize(),
                     "productVersion": STR_VERSION}
        view = self.__viewer.createView(ScreenIds.MESSAGE, viewInput)
        view.run()
        self.__model.insert('system', self.__partitioner.getSystemInformation())
        view.end()
        self.__viewer.popScreens(1)

    def initAfterLanguageSet(self):
        """
        Perform installer initialization which may show an error message to the user
        (and thus must be made after a UI language was defined)
        """
        self.__registerEventListeners()
        self.event_dispatcher.dispatchEvent("PRE_INSTALL", self.__model)

        self.__partitioner = AutoPartitionerLVM()

        # Check firmware
        self.__installer.validateOpal(self.__model)

        # Check entitlement
        self.__installer.validateEntitlement(self.__logger, self.__model)

        # obtain disk information and previous installs
        self.getDisksInformation()
        previousPkvmInstallations = self.__partitioner.getPreviousPkvmInstallations()
        self.__model.insert('previousPkvmInstallations', previousPkvmInstallations)

    def initStorageData(self):
        """
        Initialize and validate storage data required to start the
        automated installation
        """

        try:
            # get target disk devices
            disksModel = self.__model.get('disks')
            logicalVolumes = self.__model.get('logicalVolumes')
            self.__logger.info("Automated install on {}".format(disksModel))

            targetDisks = []
            targetDisksTotalSize = Size(0)
            for diskModel in disksModel:
                devDisk = self.__partitioner._getStorage().getDisk(diskModel)
                if not devDisk:
                    raise PKVMError("071")

                targetDisksTotalSize += devDisk.size
                targetDisks.append(devDisk)

            if targetDisksTotalSize < Size(MIN_DISK_SIZE):
                raise PKVMError("001")

            # standardize mount points
            for mountPoint in logicalVolumes:
                if mountPoint.endswith("/") and len(mountPoint) > 1:
                    logicalVolumes[mountPoint[:-1]] = logicalVolumes[mountPoint]
                    logicalVolumes.pop(mountPoint, None)

            # get logical volumes sizes
            lvsSizes = {}
            lvsTotalSize = Size(0)
            for lvName, lvMetadata in LVS_METADATA.items():
                if lvMetadata["mountPoint"] in logicalVolumes:
                    lvSize = logicalVolumes[lvMetadata["mountPoint"]].size
                    if Size(str(lvSize) + 'GiB') < Size(lvMetadata["minSize"] + 'GiB'):
                        raise PKVMError("059")
                else:
                    lvSize = float(lvMetadata["minSize"])

                lvsTotalSize += Size(str(lvSize) + 'GiB')
                lvsSizes[lvName] = {'size': lvSize}

            if lvsTotalSize > targetDisksTotalSize:
                raise PKVMError("058")

            self.__model.insert("lvsSizes", lvsSizes)
            self.__model.insert("targetDisks", [disk.name
                                                for disk in targetDisks])

        except PKVMError as e:
            raise
        except Exception as e:
            raise PKVMError("007")

    def initStorageDataWithPreservation(self):
        """
        Initialize and validate storage data required to start the
        automated installation with preservation of logical volumes.
        """

        logicalVolumes = self.__model.get('logicalVolumes')
        previousPkvmInstallations = self.__model.get('previousPkvmInstallations')

        # If the volume group name of all chosen logical volumes is
        # the same, then we have a PKVM 3+ installation.
        vgNames = [lv.vgname for lv in logicalVolumes.values()]
        if len(set(vgNames)) == 1:
            if vgNames[0] in previousPkvmInstallations:
                selectedPkvmInstallation = previousPkvmInstallations[vgNames[0]]
            else:
                raise PKVMError("073")
        else:
            selectedPkvmInstallation = previousPkvmInstallations[LV_PREFIX
                                                            + PKVM2_VG_ROOT]

        # check if chosen logical volumes exist in the system
        systemLvsNames = [lv.name for vg in selectedPkvmInstallation['vgs']
                          for lv in vg.lvs]
        lvsMountpoints = {lvMetadata["mountPoint"]: lvName
                          for lvName, lvMetadata in LVS_METADATA.items()}

        for lv in logicalVolumes.values():
            lvName = ""
            if lv.mountpoint in lvsMountpoints:
                lvName = lvsMountpoints[lv.mountpoint]
            else:
                if lv.mountpoint == "/var/log":
                    lvName = PKVM2_LV_LOG
            if lv.vgname + '-' + LV_PREFIX + lvName not in systemLvsNames:
                raise PKVMError("063")

        vgName = selectedPkvmInstallation['rootVg'].name
        self.__model.insert('vgName', vgName)
        self.__model.insert('selectedPkvmInstallation',
                            selectedPkvmInstallation)

        self.__partitioner.setTargetLVMInfo(selectedPkvmInstallation['vgs'])
        disks = self.__partitioner.getAffectedDisks()
        self.__model.insert('targetDisks', disks)


    def splashScreen(self, params):
        """
        Displays installer splash screen to the user

        Args:
            params (dict): screen parameters
        """

        viewInput = {"productVersion": STR_VERSION}
        self.__viewer.runView(ScreenIds.SPLASH, viewInput)

        return ScreenIds.SELECT_LANGUAGE, {}

    def setLanguageScreen(self, params):
        """
        Displays the language screen to the user

        Returns:
            str: next screen ID
        """

        languages = i18n.getAvailableLanguages()

        # get menu screen instance
        viewInput = {"languages": languages, "productVersion": STR_VERSION}
        viewOutput = self.__viewer.runView(ScreenIds.SELECT_LANGUAGE, viewInput)
        language = viewOutput["language"]
        i18n.setLanguage(language)
        self.__model.insert("language", language)

        if not self.__initialized:
            self.initAfterLanguageSet()
            self.__initialized = True

        return ScreenIds.LICENSE, {}


    def licenseScreen(self, _):
        """
        Handles the License screen

        Returns:
            str: next screen ID
        """
        self.__logger.info('License screen')

        language = self.__model.get("language")

        # display the IBM license in the selected language
        viewInput = {"license": getIBMLicense(language, self.__logger), "accept": False,
                       "productVersion": STR_VERSION}
        viewOutput  = self.__viewer.runView(ScreenIds.LICENSE, viewInput)
        if not viewOutput["accepted"]:
            return self.__viewer.popPrevScreen(), {}

        viewInput = {"license": getNonIBMLicenseNotices(self.__logger), "accept": True,
                       "productVersion": STR_VERSION}
        viewOutput = self.__viewer.runView(ScreenIds.LICENSE, viewInput)
        if not viewOutput["accepted"]:
            self.__logger.info('Non IBM licenses and notices declined, rebooting')
            self.rebootSystem(True)

        self.__model.insert('licenseAccepted', True)

        # decide next screen
        self.__viewer.popScreens(2)
        return ScreenIds.MENU, {}


    def menuScreen(self, params):
        """
        Handles the Menu Install screen

        Returns:
            str: next screen ID
        """
        self.__logger.info('Menu screen')

        # options to be configured on menu screen
        menuOptions = []

        # install option
        menuOptions.append((INSTALL_IBM_POWERKVM.localize(), "singleinstall"))

        if self.__model.get('previousPkvmInstallations'):
            menuOptions.append((INSTALL_PRESERVING_LVS.localize(), "preserve"))

        # get menu screen instance
        viewInput = {"options": menuOptions, "productVersion": STR_VERSION}
        viewOutput = self.__viewer.runView(ScreenIds.MENU, viewInput)
        res = viewOutput["pressedButton"]

        if res == "ok":
            installationType = viewOutput["installationType"]
            self.__model.insert('preserveData', installationType == "preserve")

        # decide next screen
        if res == "back":
            nextScreen = self.__viewer.popPrevScreen()
        else:
            preserveData = self.__model.get('preserveData')
            if preserveData:
                nextScreen = ScreenIds.SELECT_PREV_INSTALL
            else:
                nextScreen = ScreenIds.SELECT_DISKS

        return nextScreen, {}

    def getDisksSize(self, disks, blivetDisks):
        """
        Get total size of disks

        Args:
            disks ([str]): disks
            blivetDisks (Blivet:DiskDevice): all disks in the system

        Returns:
            int: total size of disks
        """

        totalSize = Size(0)
        for blivetDisk in blivetDisks:
            for disk in disks:
                if blivetDisk.name == disk:
                    totalSize += blivetDisk.size

        return totalSize

    def selectPrevInstallationScreen(self, params):
        """
        Handles the previous installation selection screen

        Args:
            params (dict): Parameters that came from the previous screen
        """

        previousPkvmInstallations = self.__model.get('previousPkvmInstallations')
        viewInput = {"previousPkvmInstallations": previousPkvmInstallations}
        viewOutput = self.__viewer.runView(ScreenIds.SELECT_PREV_INSTALL, viewInput)

        res = viewOutput["pressedButton"]
        if res == 'back':
            currScreen = self.__viewer.popPrevScreen()
        else:
            currScreen = ScreenIds.SETUP_ROOT_PASSWORD
            self.__model.insert('vgName', viewOutput["selectedVGName"])

            selectedPkvmInstallation = previousPkvmInstallations[viewOutput["selectedVGName"]]
            self.__model.insert('selectedPkvmInstallation', selectedPkvmInstallation)
            self.__partitioner.setTargetLVMInfo(selectedPkvmInstallation['vgs'])

            disks = self.__partitioner.getAffectedDisks()
            self.__model.insert('targetDisks', disks)

        return currScreen, {}

    def setupLvmScreen(self, params):
        """
        Handles the LVM setup screen
        """

        self.__logger.info('Alter logical volume size screen')

        blivetDisks = self.__model.get('system')["disks"]
        targetDisks = self.__model.get('targetDisks')
        targetDisksSize = self.getDisksSize(targetDisks, blivetDisks)
        viewInput = {"targetDisksTotalSize": targetDisksSize}
        viewOutput = self.__viewer.runView(ScreenIds.SETUP_LVM, viewInput)
        res = viewOutput["pressedButton"]
        self.__model.insert('vgName', viewOutput["vgName"])
        self.__model.insert('lvsSizes', viewOutput["lvsSizes"])
        if res == 'back':
            currScreen = self.__viewer.popPrevScreen()
        else:
            currScreen = ScreenIds.SETUP_ROOT_PASSWORD

        return currScreen, {}

    def selectDisksScreen(self, params):
        """
        Handles the Disk Selection screen
        """
        try:
            blivetSystem = self.__model.get('system')
            blivetStorageDevices = blivetSystem["devices"]
            blivetDisks = blivetSystem["disks"]
            blivetPartitions = blivetSystem["partitions"]
            disks = []
            for blivetDisk in blivetDisks:
                disk = {}
                disk["name"] = blivetDisk.name
                if blivetDisk.type == 'dm-multipath':
                    disk["serial"] = blivetDisk.parents[0].serial
                    disk["multipath"] = True
                else:
                    disk["serial"] = blivetDisk.serial
                    disk["multipath"] = False
                disk["size"] = str(blivetDisk.size)
                disks.append(disk)

            # no disk retrieved: log the error and exit
            if len(disks) == 0:
                # Straight back to main menu
                raise PKVMError("002")

            viewInput = {"systemDisks": disks, "productVersion": STR_VERSION}
            view = self.__viewer.createView(ScreenIds.SELECT_DISKS, viewInput)
            validInput = False
            targetDisks = []
            while not validInput:
                viewOutput = view.run(targetDisks)
                action = viewOutput["action"]
                if action == "back":
                    break
                elif action == "details":
                    diskName = viewOutput["diskName"]
                    for blivetDisk in blivetDisks:
                        if blivetDisk.name == diskName:
                            diskInfo = self.createDiskMetadata(blivetDisk, blivetStorageDevices, blivetDisks, blivetPartitions)
                            view.showDiskInformation(diskInfo)
                            break
                    continue
                elif action == "ok":
                    targetDisks = viewOutput["selectedDisks"]
                    blivetSelectedDisks = []
                    for blivetDisk in blivetDisks:
                        if blivetDisk.name in targetDisks:
                            blivetSelectedDisks.append(blivetDisk)
                    validInput, error = self.__model.validateTargetDisks(blivetSelectedDisks)
                    if not validInput:
                        targetDisks = []
                        if error == "no_disks":
                            view.showErrorNoDiskSelected()
                        elif error == "insufficient_disk_space":
                            view.showErrorDiskSize()
                        continue

                    targetDisksStr = " ".join(targetDisks)
                    self.__logger.info('Disks selected {}'.format(targetDisksStr))

                    self.__model.insert("targetDisks", targetDisks)

        except PKVMError as e:
            raise
        except Exception as e:
            raise PKVMError("008")

        # decide next screen
        if action == "back":
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.SETUP_LVM
        return nextScreen, {}


    def setupRootPasswordScreen(self, params):
        """
        Handles the Root Change Password screen
        """
        self.__logger.info('Root Password Change screen')

        validInput = False
        # create root password selection screen
        view = self.__viewer.createView(ScreenIds.SETUP_ROOT_PASSWORD)

        while not validInput:
            # run root password selection screen UI
            viewOutput = view.run()
            res = viewOutput["pressedButton"]
            passwd = viewOutput["passwd"]
            if res == 'back':
                break

            # validate password
            if not self.__model.validatePassword(passwd):
                view.showErrorLength()
                continue

            validInput = True

        if res == 'ok':
            # hash passwd for shadow file (sha512)
            hashPasswd = crypt.crypt(passwd, '$6$')
            self.__model.insert('pass', hashPasswd)
            self.__model.insert('isCrypted', True)
            self.__logger.info('Root password changed successfuly')

        # decide next screen
        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.SELECT_TIMEZONE
        return nextScreen, {}


    def selectTimezoneScreen(self, params):
        """
        Handles the timezone screen
        """
        self.__logger.info('Adjust Timezone screen')

        # get timezone module
        timezone = Timezone()

        # call the window passing a list with all timezones available
        viewInput = {"timezones": timezone.getEntries()}
        viewOutput = self.__viewer.runView(ScreenIds.SELECT_TIMEZONE, viewInput)
        res = viewOutput["pressedButton"]
        zone = viewOutput["timezone"]
        utc = viewOutput["utc"]

        # setup the selected timezone/utc
        if res == 'ok':
            self.__model.insert('tz', zone)
            self.__model.insert('isUTC', utc)
            timezone.setTimezone(zone)
            timezone.writeConfig()
            timezone.setUTC(utc)
            self.__logger.info('Timezone: %s, UTC: %s' % (zone, str(utc)))

        # decide next screen
        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.SETUP_NTP
        return nextScreen, {}


    def listNwInterfacesScreen(self, params):
        """
        Handles the list of network ifaces to be configured
        """
        self.__logger.info('listNetworkIfaces screen')

        # get network interfaces data
        nwInterfaces = Network.getAvailableInterfaces()
        activeNwInterfaces = Network.getLinkedInterfaces()

        # call the window
        if nwInterfaces:
            viewInput = {"nwInterfaces": nwInterfaces,
                         "activeNwInterfaces": activeNwInterfaces}
            viewOutput = self.__viewer.runView(ScreenIds.LIST_NETWORK_INTERFACES, viewInput)
            res = viewOutput["pressedButton"]
            eth = viewOutput["selectedNwInterfaceName"]
            macaddr = viewOutput["selectedNwInterfaceMacAddress"]
        else:
            self.__logger.info("No network interfaces were found on this" \
                               "system. Skipping to next screen.")
            res = 'ok'

        # decide next screen
        if res == "back":
            nextScreenParams = {}
            nextScreen = self.__viewer.popPrevScreen()
        elif res == 'ok' or not eth or not nwInterfaces:
            # proceed to next screen
            self.__logger.info('Network configuration go to NEXT screen')
            nextScreenParams = {}
            nextScreen = ScreenIds.SETUP_DNS
        elif res is None or res == "edit":
            # configure the network for the selected iface. The check for None
            # is used so the user can edit network interface details when
            # clicking on it instead of "edit" button.
            nextScreenParams = {"name": eth, "macAddress": macaddr}
            nextScreen = ScreenIds.SETUP_NETWORK_INTERFACE
        return nextScreen, nextScreenParams

    def setupNwInterfaceScreen(self, params):
        """
        Configures the network interface selected

        Args:
            params (dict): net iface to be configured
        """
        self.__logger.info('configNetwork screen')

        nwInterface = NetworkInterface(params["name"])
        activeNwInterfaces= Network.getLinkedInterfaces()

        # call the window
        viewInput = {"networkInterfaceName": params["name"],
                     "networkInterfaceMacAddress": params["macAddress"],
                     "networkInterface": nwInterface,
                     "activeNwInterfaces": activeNwInterfaces}
        viewOutput = self.__viewer.runView(ScreenIds.SETUP_NETWORK_INTERFACE, viewInput)
        res = viewOutput["pressedButton"]
        networkData = viewOutput["network"]

        if res == 'ok':

            self.__saveNwInterfaceConfig(networkData, nwInterface)

            # make changes before saving to model
            if networkData["dhcp"]:
                networkData['bootProto'] = 'dhcp'
            else:
                networkData['bootProto'] = 'none'
            networkData['hostname'] = ''
            networkData['nodefroute'] = False
            # @IMPROVEMENT: this attrib exists only to keep backwards compatibility with the
            # rest of the code and should be removed
            networkData['device'] = networkData["name"]

            data = []

            # append the network data in the list if
            # there is already another device to be
            # configured
            if self.__model.get('network'):
                data = self.__model.get('network')
                # ensure only one entry per NIC on network list
                for i in data:
                    if i.get('device') == networkData.get('device'):
                        data.remove(i)
            data.append(networkData)
            self.__model.insert('network', data)
            self.__logger.info(str(data))

        # decide next screen
        return self.__viewer.popPrevScreen(), {}

    def setupDnsScreen(self, params):
        """
        Configures the DNS
        """
        self.__logger.info('dnsSetup screen')

        # call the window
        if self.__model.hasKey('hostname'):
            hostname = self.__model.get('hostname')
        else:
            hostname = reverseDNSLookup()

        if self.__model.hasKey('dns'):
            dnsData = self.__model.get('dns')
        else:
            dnsData = {}
            systemDnsManager = DNS()
            dnsData["dnsList"] = systemDnsManager.getNameservers()
            dnsData["search"] = systemDnsManager.getSearchList()

            if dnsData["search"] == "":
                dnsData["search"] = searchDNSFromFQDN(socket.getfqdn(hostname))

        viewInput = {"dnsData": dnsData, "hostname": hostname}
        viewOutput = self.__viewer.runView(ScreenIds.SETUP_DNS, viewInput)
        res = viewOutput["pressedButton"]
        dnsData = viewOutput["dnsData"]
        hostname = viewOutput["hostname"]

        # store the dns data
        if res == 'ok':
            self.__model.insert('dns', dnsData)
            # Set hostname on ISO so that LVs will display 'creation_host'
            # variable with the hostname given to the installed system.
            # Other tools might also depend on this.
            subprocess.call(["hostnamectl", "set-hostname", hostname])
            # get hostname trimmed by hostnamectl
            self.__model.insert('hostname', socket.gethostname())

            self.__logger.info(str(dnsData))

        # decide next screen
        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.INSTALLATION_SUMMARY
        return nextScreen, {}

    def setupNtpScreen(self, params):
        """
        Configures the NTP
        """
        self.__logger.info('NTPsetup screen')

        # call the window
        viewOutput = self.__viewer.runView(ScreenIds.SETUP_NTP)
        res = viewOutput["pressedButton"]
        ntpData = viewOutput["ntpData"]

        # store the dns data
        if res == 'ok':
            self.__model.insert('ntpservers', ntpData)
            self.__logger.info("NTP server list: %s" % str(ntpData))

        # decide next screen
        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.SETUP_DATETIME
        return nextScreen, {}


    def setupDateTimeScreen(self, params):
        """
        Configures the Date time
        """
        self.__logger.info('DateTime screen')
        timezone = Timezone()

        # call the window
        viewOutput = self.__viewer.runView(ScreenIds.SETUP_DATETIME)
        res = viewOutput["pressedButton"]
        rdata = viewOutput["datetime"]
        if res == 'ok':
            timezone.setDateTime(rdata)
            self.__model.insert('datetime', rdata)

            date_str = rdata["year"] + "/" + rdata["month"] + "/" + rdata["day"] + " " + \
                rdata["hour"] + ":" + rdata["minute"] + ":" + rdata["second"]

            self.__logger.info('Date/time inputted: %s' % date_str)

        # decide next screen
        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.LIST_NETWORK_INTERFACES
        return nextScreen, {}

    def summaryScreen(self, params):
        """
        Show summary of the installation

        Returns:
            str: next screen ID
        """
        self.__logger.info('Summary')

        # call the window
        viewInput = {"installData": self.__model, "productVersion": STR_VERSION}
        viewOutput = self.__viewer.runView(ScreenIds.INSTALLATION_SUMMARY, viewInput)
        res = viewOutput["pressedButton"]

        if res == 'back':
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.CONFIRM_INSTALLATION
        return nextScreen, {}

    def confirmInstallationScreen(self, params):

        viewInput = {"data": self.__model, "productVersion": STR_VERSION}
        viewOutput = self.__viewer.runView(ScreenIds.CONFIRM_INSTALLATION, viewInput)
        if viewOutput["pressedButton"] == "back":
            nextScreen = self.__viewer.popPrevScreen()
        else:
            nextScreen = ScreenIds.INSTALLATION_PROGRESS

        return nextScreen, {}

    def installationProgressScreen(self, _):
        """
        Handles the installation progress screen

        """
        self.__logger.info('InstallProgress screen')

        disks = self.__model.get('targetDisks')
        vgName = self.__model.get('vgName')
        preserveData = self.__model.get('preserveData')

        # get the installation progress view
        viewInput = {"productVersion": STR_VERSION}
        install = self.__viewer.createView(ScreenIds.INSTALLATION_PROGRESS, viewInput)
        install.updateProgress(FORMATTING_DISKS.localize(), 0)

        # format disks and create partitions and LVM (volumes groups and logical volumes)
        try:
            self.__logger.info('Formatting disks...')

            self.__partitioner.setSystemVGName(vgName, not preserveData)

            if not preserveData:
                lvsSizes = self.__model.get('lvsSizes')
                self.__partitioner.setupAutoLVM(disks, lvsSizes)
            else:
                selectedPkvmInstallation = self.__model.get('selectedPkvmInstallation')
                prevInstallPartitions = selectedPkvmInstallation['partitions']
                self.__partitioner.reuseCurrentAutoLVM(prevInstallPartitions)

            self.__partitioner.commit(not preserveData)
            self.__partitioner.mountPartitions(self.__model.get('mountDir'))

        # some error handling disks: abort
        except PKVMError as e:
            raise
        except Exception as e:
            raise PKVMError("004")

        try:
            disksStr = " ".join(disks)


            if preserveData and selectedPkvmInstallation['version'] == 3:
                self.__partitioner.cleanupVar()

            # install packages
            self.__logger.info("Installing IBM PowerKVM into disk %s..." % disksStr)
            install.updateProgress(INSTALLING_IBM_POWERKVM_INTO_DISK.localize() % disksStr, 25)
            targetDir = self.__model.get('mountDir')
            self.__installer.installPackages(targetDir, install.installInfo)

            # perform post-install actions
            self.__installer.doPostInstall(targetDir, self.__partitioner)

        except PKVMError as e:
            install.updateProgress(ERROR_WHILE_INSTALLING_IBM_POWERKVM.localize() % disksStr, 30)
            raise
        except Exception as e:
            install.updateProgress(ERROR_WHILE_INSTALLING_IBM_POWERKVM.localize() % disksStr, 30)
            raise PKVMError("005")

        try:
            # notifies EVT_POST_INSTALL
            self.__logger.info('System Installed. Starting post install scripts.')
            install.updateProgress(SYSTEM_INSTALLED_STARTING_POST.localize(), 100)
            self.event_dispatcher.dispatchEvent("POST_INSTALL", self.__model)
        except PKVMError as e:
            raise
        except Exception as e:
            raise PKVMError("006")

        return ScreenIds.REBOOT, {}


    def rebootSystem(self, error=False):
        """
        Handles the Reboot system screen

        Args:
            error (bool): reboot due to error
        """
        self.__logger.info('RebootSystem screen')

        viewInput = {"error": error, "productVersion": STR_VERSION}
        if error:
            self.__viewer.runView(ScreenIds.REBOOT, viewInput)
        elif self.__model.get('isManual'):
        # Only show reboot message if NOT in auto mode
            # message user about rebooting
            self.__viewer.runView(ScreenIds.REBOOT, viewInput)

        # reboot
        subprocess.call("reboot")

    def __getFirstChild(self, blivetStorageDevices, blivetStorageDevice):
        """
        Returns the first Blivet device name that depends on a Blivet device.

        Args:
            blivetStorageDevices ([Blivet.Device]):  Blivet devices in system
            blivetStorageDevice (Blivet.Device):  Blivet partition

        Returns:
            str: device name, empty string if none is found
        """
        children = [ children for children in blivetStorageDevices \
                    if blivetStorageDevice in children.parents ]

        # if there is a children, return the first child
        if children:
            return children[0].name

        return ''

    def createDiskMetadata(self, blivetDisk, blivetStorageDevices, blivetDisks, blivetPartitions):
        """
        Create disk metadata string

        Args:
            blivetDisk (Blivet.Device):  Blivet disk device
            blivetStorageDevices ([Blivet.Device]): list of storage devicesoo
            blivetDisks ([Blivet.Device]):  Blivet disk devices in system
            blivetPartitions ([?]): Blivet partitions in a system

        Returns:
            dict: disk metadata. Valid keys: name, mpathSlaves, serial, size, partitions.
        """

        disk = {}
        disk["name"] = blivetDisk.name
        if blivetDisk.type == 'dm-multipath':
            disk["mpathSlaves"] = [ d.name for d in blivetDisk.parents ]
            disk["serial"] = blivetDisk.parents[0].serial
        else:
            disk["serial"] = blivetDisk.serial
        disk["size"] = str(blivetDisk.size)

        partitionsDicts = []
        for partition in blivetPartitions[blivetDisk.name]:

            self.__logger.debug("processing partition %s" % partition.name)

            # get the format partition
            extraInfo = ''
            format = ' '
            if partition.format.type is not None:
                format = partition.format.name

                # get the raid or vg name if necessary
                if partition.format.type == 'lvmpv' or \
                   partition.format.type == 'mdmember':
                    extraInfo = self.__getFirstChild(blivetStorageDevices, partition)

            if partition.isExtended:
                format = 'extended'

            partitionDict = {"name": partition.name,
                      "size": str(partition.size),
                      "format": format,
                      "extraInfo": extraInfo}
            partitionsDicts.append(partitionDict)
        disk["partitions"] = partitionsDicts

        return disk
