#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


class NoEventListenersError(Exception):
    pass

class EventDispatcher(object):
    """
    Implementation of an event dispatcher.

    Attributes:
        events_callbacks ({str : callable}): Dictionary that maps
            event identifiers to the callback functions that should be
            executed when a given event happens.
    """
    def __init__(self):
        self.events_callbacks = {}

    def register(self, event_identifier, callback):
        """
        Registers a callback function to the event's listeners list.

        Args:
            event_identifier (str): some string that identifies an
                event.
            callback (function): function to be called when event is
                dispatched

        Raises:
            TypeError: if ``event_identifier`` is not a string or if
                ``callback`` is not a callable object.
        """
        if not isinstance(event_identifier, basestring):
            raise TypeError("event identifier must be a string.")

        if not callable(callback):
            raise TypeError("callback parameter should be a callable object.")

        if event_identifier not in self.events_callbacks:
            self.events_callbacks[event_identifier] = [callback]
        else:
            self.events_callbacks[event_identifier].append(callback)

    def dispatchEvent(self, event_identifier, *args, **kwargs):
        """
        Executes all the callback's registered for the given event.

        Args:
            event_identifier (str): string that identifies the event
                to be dispatched.
            *args ([?]): all args provided to the method.
            **kwargs ({?:?}): all named args provided to the method.

        Raises:
            NoEventListenersError: if there are no callbacks registered
                for the event being dispatched.
        """
        if event_identifier not in self.events_callbacks:
            raise NoEventListenersError("no listeners for \'%s\' event." \
                                       % event_identifier)

        for cb in self.events_callbacks[event_identifier]:
            cb(*args, **kwargs)
