#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from subprocess import Popen, PIPE
import logging
import os
import shutil
import traceback

from controller.pkvmerror import PKVMError
from model.config import POWERKVM_LOG_DIR
from modules.scriptbase import ScriptBase


#
# CODE
#
class ExecPostScripts(ScriptBase):
    """
    Execute post scripts provided in the kickstart file
    """

    def __init__(self):
        """
        Contructor.
        """
        self.__logger = logging.getLogger(__name__)

    def onPostInstall(self, model):
        """
        Handles the post install event. Creates a mark file on
            installed and copy the log file from live image to the
            installed system.

        Args:
            model (Model): installation data
        """
        try:
            self.__logger.info("Executing ExecPostScripts module (EVT_POST_INSTALL).")
            # get root directory previously mounted
            rootMountedDir = model.get('mountDir')
            # root directory not mounted: abort
            if rootMountedDir is None:
                self.__logger.critical("No mounting point.")
                raise PKVMError("026")
            # create target log directory
            targetLogDir = rootMountedDir + POWERKVM_LOG_DIR
            if not os.path.exists(targetLogDir):
                os.makedirs(targetLogDir)

            postScriptKS = POWERKVM_LOG_DIR + '/%spostScriptKS.sh'
            postScriptKSLog = POWERKVM_LOG_DIR + '/%spostScriptKS.log'
            postScriptKSErr = POWERKVM_LOG_DIR + '/%spostScriptKS.err'
            counter = 0
            for script in model.get('postScripts'):
                fileName = postScriptKS % counter
                systemFileName = rootMountedDir + fileName
                tempFileName = fileName
                copyFileName = systemFileName
                if script['inChroot']:
                    tempFileName = systemFileName
                    copyFileName = fileName

                # create post script
                with open(tempFileName, "w") as fd:
                    fd.write(script['script'])
                os.chmod(tempFileName, 0777)
                shutil.copy(tempFileName, copyFileName)

                # execute post script
                cmdPost = [script['interp'], fileName]
                if script['inChroot']:
                    cmdPost.insert(0, rootMountedDir)
                    cmdPost.insert(0, 'chroot')
                proc = Popen(cmdPost, stdout=PIPE, stderr=PIPE)
                out, err = proc.communicate()
                # create post script log
                if out:
                    # log in the live image
                    logFileName = postScriptKSLog % counter
                    systemLogFileName = rootMountedDir + logFileName
                    with open(logFileName, "w") as fd:
                        fd.write(out)
                    # copy live image log to installed system
                    shutil.copy(logFileName, systemLogFileName)

                if proc.returncode != 0:
                    self.__logger.critical("Failed to execute kickstart post-script (%s) (exit code = %s):\n%s" % (cmdPost, proc.returncode, err))
                    self.__logger.critical("Check kicktart post-scrit log file: %s" % logFileName)
                    # err in the live image
                    errFileName = postScriptKSErr % counter
                    systemErrFileName = rootMountedDir + errFileName
                    with open(errFileName, "w") as fd:
                        fd.write("Failed to execute kickstart post-script (%s):\n%s (exit code = %s)\n" % (fileName, cmdPost, proc.returncode))
                        if err:
                            fd.write(err)
                    # copy live image err to installed system
                    shutil.copy(errFileName, systemErrFileName)
                    self.__logger.critical("Check kicktart post-scrit error file: %s" % errFileName)

                counter += 1
        except PKVMError as e:
            raise
        except Exception as e:
            raise PKVMError("024")
