#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import crypt
import logging
import os
import pprint
import re
import shutil
import urllib

from controller.pkvmerror import PKVMError
from model.config import KICKSTART_FILE
from model.config import POWERKVM_LOG_DIR
from model.network import reverseDNSLookup
from model.disk import mount, umount
from modules.i18n.i18n import setLanguageKS
from modules.installauto.pkvmkickstart.parser import PkvmKickstartParser
from modules.network.network import Network
from modules.scriptbase import ScriptBase
from socket import gethostname
from subprocess import Popen, PIPE
from subprocess import call


#
# CONSTANTS
#
CMD_FILE = '/proc/cmdline'
DEFAULT_KICKSTART_FILE = '/opt/ibm/powerkvm/kickstart/manufacture.ks'
REPO_PATH = 'file:///media/packages/'
ROOT_PASS = '$6$CN6lwYuX$CRY7I4gNcygJeMytuDPsZW7ODUyoE5e3HDztHznxnSK5d8hNGe3MjLo6JTAUMfzcSpsIcwvhJ1euKDVEXk.8n1'
TIME_ZONE = 'America/New_York'
LANG = 'en_US'


#
# CODE
#
class InstallAuto(ScriptBase):
    """
    Parser data required for auto installation.
    """

    def __init__(self):
        """
        Contructor
        """
        self.__model = {}
        self.__resetData()
        self.__kickstartFile = ''
        self.__logger = logging.getLogger(__name__)

    def __resetData(self):
        """
        Reset data to default values.
        """
        self.__model = {
            'action': None,
            'repo': REPO_PATH,
            'lang': LANG,
            'disk': None,
            'datapoll': None,
            'pass': ROOT_PASS,
            'isCrypted': True,
            'tz': TIME_ZONE,
            'isUTC': False,
            'network': [],
            'preScripts': [],
            'postScripts': [],
            'isKickstart': False,
            'ntpservers': [],
        }

    def __downloadHttpFile(self):
        """
        Download config file for auto install via HTTP.
        """
        try:
            urllib.urlretrieve(self.__kickstartFile, KICKSTART_FILE)
            self.__kickstartFile = KICKSTART_FILE
        except Exception:
            raise PKVMError("018")

    def __downloadTftpFile(self):
        """
        Download config file for auto install via TFTP.
        """
        tftpServer = self.__kickstartFile[7:].split('/')[0]
        serverLen = len(tftpServer)
        tftpFile = self.__kickstartFile[7 + serverLen + 1:]
        proc = Popen(['tftp', tftpServer, '-c', 'get', tftpFile,
                      KICKSTART_FILE], stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            self.__logger.critical(
                "Failed to download (TFTP) install auto file(%s) (exit code = "
                "%s):", self.__kickstartFile, proc.returncode)
            self.__logger.critical("%s", err)
            raise PKVMError("019")
        self.__kickstartFile = KICKSTART_FILE

    def __downloadNfsFile(self):
        """
        Download config file for auto install via NFS.
        """
        try:
            nfsFile = self.__kickstartFile.split('/')[-1]
            nfsSections = self.__kickstartFile.split(':')
            if len(nfsSections) == 4:
                nfsDir = self.__kickstartFile.split(':')[3][:-len(nfsFile)]
                nfsOpts = self.__kickstartFile.split(':')[1]
                nfsServer = self.__kickstartFile.split(':')[2]
            else:
                nfsDir = self.__kickstartFile.split(':')[2][:-len(nfsFile)]
                nfsServer = self.__kickstartFile.split(':')[1]
                nfsOpts = ""
            if nfsOpts != "":
                nfsMount = "-o " + nfsOpts + " " + nfsServer + ":" + nfsDir
            else:
                nfsMount = nfsServer + ":" + nfsDir
            nfsMount = mount(nfsMount)
            nfsFile = os.path.join(nfsMount, nfsFile)
            shutil.copy(nfsFile, KICKSTART_FILE)
            umount(nfsMount)
            self.__kickstartFile = KICKSTART_FILE
        except Exception:
            raise PKVMError("020")

    def __execScript(self):
        """
        Execute pre script file(s) from kickstart.
        """
        try:
            self.__logger.info("Executing pre-script from kickstart!")
            #
            # kickstart pre scripts
            #
            preScriptKS = POWERKVM_LOG_DIR + '/%spreScriptKS.sh'
            preScriptKSLog = POWERKVM_LOG_DIR + '/%spreScriptKS.log'
            counter = 0
            for script in self.__model.get('preScripts'):
                fileName = preScriptKS % counter
                logFileName = preScriptKSLog % counter

                # create pre script
                with open(fileName, "w") as fd:
                    fd.write(script['script'])
                os.chmod(fileName, 0777)

                # execute pre script
                cmdPre = [script['interp'], fileName]
                proc = Popen(cmdPre, stdout=PIPE, stderr=PIPE)
                out, err = proc.communicate()
                # create pre script log
                if out:
                    with open(logFileName, "w") as fd:
                        fd.write(out)

                if proc.returncode != 0:
                    self.__logger.critical(
                        "Failed to execute kickstart pre-script (%s) (exit "
                        "code = %s): %s", cmdPre, proc.returncode, err)
                    raise PKVMError("023")

                counter += 1

        except PKVMError:
            raise
        except Exception:
            raise PKVMError("086")

    def __setLanguage(self, lang):
        """
        Sets the language

        Args:
            lang (str): language
        """

        if lang:
            self.__model['lang'] = lang

        self.__logger.debug(
            "Language to be set by kickstart: %s", self.__model.get('lang'))

        langSet = setLanguageKS(self.__model.get('lang'))

        self.__logger.debug("Language effectively set: %s", langSet)

    def __setLvmInformation(self, volumeGroups, partitions, logicalVolumes):
        """
        Sets the LVM information

        Args:
            volumeGroups ([?]): data from ks volgroup lines
            partitions ([?]): data from ks part lines
            logicalVolumes ([?]): data from ks logvol lines
        """

        if not self.__model['preserveData']:
            self.__model['vgName'] = vgName = volumeGroups[0].vgname

            physicalVolumes = volumeGroups[0].physvols
            self.__model['disks'] = [part.disk for part in partitions
                                     if part.mountpoint in physicalVolumes]

        lvs = {lv.mountpoint : lv for lv in logicalVolumes}
        self.__model['logicalVolumes'] = lvs

    def __setPassword(self, rootPw):
        """
        Sets the root password and encrypt it if necessary

        Args:
            rootPw (dict): password and encryption data.
        """

        if rootPw["isCrypted"]:
            self.__logger.debug("Using encrypted password!")
            self.__model['pass'] = rootPw["password"]
        else:
            # hash passwd for shadow file (sha512)
            self.__logger.debug("Encrypting password...")
            self.__model['pass'] = \
                crypt.crypt(rootPw["password"], '$6$')
        # Note tha even 'isCrypted' is set to false, starting at this
        # poin # password provided in kickstart will be encrypted for
        # the sake of security
        self.__model['isCrypted'] = rootPw["isCrypted"]

    def __setTimezone(self, timezoneData):
        """
        Sets the timezone data

        Args:
            timezoneData (dict): timezone, utc and ntp data
        """

        self.__model['tz'] = timezoneData["timezone"]
        self.__model['isUTC'] = timezoneData["isUTC"]
        self.__model['nontp'] = timezoneData["nontp"]
        self.__model['ntpservers'] = timezoneData["ntpservers"]

    def __setNetworkInformation(self, networks):
        """
        Set network information

        Args:
            networks ([dict]): network data
        """

        for net in networks:
            info = {}
            info['bootProto'] = net.bootProto
            info['device'] = net.device
            info['ip'] = net.ip
            info['netmask'] = net.netmask
            info['gateway'] = net.gateway
            info['nameserve'] = net.nameserver
            info['nodefroute'] = net.nodefroute
            info['bridge'] = False
            info['onboot'] = net.onboot
            self.__model['network'].append(info)

            # Set hostname on livecd
            if net.hostname == 'localhost':
                #@TODO This should be refactor into the network module.
                # It should handle all these network tasks and be used
                # in the same manner in both manual and automatic install.
                hostname = reverseDNSLookup()
            else:
                hostname = net.hostname

            call(['hostnamectl', 'set-hostname', hostname])
            self.__model['hostname'] = hostname

    def __preParserFile(self, data):
        """
        Pre parse kickstart file ignoring '%include' parameter.
            Execute the pre-script as well.

        Args:
            data (str): kickstart file content
        """
        # reset any previous data
        self.__resetData()

        try:
            self.__logger.info("Pre-parsing kickstart file")
            pkvmKsParser = PkvmKickstartParser(False)
            pkvmKsParser.readKickstartFromString(data)

            if pkvmKsParser.preScripts:
                for script in pkvmKsParser.preScripts:
                    info = {}
                    info['script'] = script.script
                    info['interp'] = script.interp
                    info['inChroot'] = script.inChroot
                    info['errorOnFail'] = script.errorOnFail
                    self.__model['preScripts'].append(info)
                self.__execScript()
        except PKVMError:
            raise
        except Exception:
            raise PKVMError("080")

    def __parserFile(self, data):
        """
        Parse automated installation (kickstart) file

        Args:
            data (str): kickstart file content
        """
        # reset any previous data
        self.__resetData()

        try:
            self.__logger.info("Parsing kickstart file")
            pkvmKsParser = PkvmKickstartParser()
            pkvmKsParser.readKickstartFromString(data)

            self.__setLanguage(pkvmKsParser.lang)
            # None or True means not preserve
            self.__model['preserveData'] = pkvmKsParser.formatAllLVs is False
            self.__setLvmInformation(pkvmKsParser.volumeGroups, pkvmKsParser.partitions,
                                     pkvmKsParser.logicalVolumes)

            if pkvmKsParser.rootpw["password"]:
                self.__setPassword(pkvmKsParser.rootpw)

            if pkvmKsParser.tzMap["timezone"]:
                self.__setTimezone(pkvmKsParser.tzMap)

            if pkvmKsParser.networks:
                self.__setNetworkInformation(pkvmKsParser.networks)

            if pkvmKsParser.preScripts:
                for script in pkvmKsParser.preScripts:
                    info = {}
                    info['script'] = script.script
                    info['interp'] = script.interp
                    info['inChroot'] = script.inChroot
                    info['errorOnFail'] = script.errorOnFail
                    self.__model['preScripts'].append(info)

            if pkvmKsParser.postScripts:
                for script in pkvmKsParser.postScripts:
                    info = {}
                    info['script'] = script.script
                    info['interp'] = script.interp
                    info['inChroot'] = script.inChroot
                    info['errorOnFail'] = script.errorOnFail
                    self.__model['postScripts'].append(info)
        except PKVMError:
            raise
        except Exception:
            raise PKVMError("021")

    def __ksParser(self):
        """
        Parse automated installation (kickstart) file
        """
        # reset any previous data
        self.__resetData()

        try:
            self.__model['isKickstart'] = True
            if self.__kickstartFile == 'default':
                self.__kickstartFile = DEFAULT_KICKSTART_FILE
            else:
                # download install auto file if needed
                if (self.__kickstartFile.startswith('http') or
                        self.__kickstartFile.startswith('ftp')):
                    self.__downloadHttpFile()
                if self.__kickstartFile.startswith('tftp'):
                    self.__downloadTftpFile()
                if self.__kickstartFile.startswith('nfs'):
                    self.__downloadNfsFile()

            # check if kickstart is not empty
            if os.stat(self.__kickstartFile).st_size == 0:
                raise PKVMError("079")

            # parse the file and fill the dictionary
            with open(self.__kickstartFile) as fd:
                data = fd.read()

            self.__logger.debug("Kickstart file => BEGIN")
            self.__logger.debug("Kickstart content:")
            for line in data.split("\n"):
                # hide password in case of not encrypted
                if line.lstrip(" ").startswith("rootpw"):
                    line = re.sub(' +', ' ', line.lstrip(" "))
                    if len(line.split(" ")) == 2:
                        line = "rootpw XXXXX"
                self.__logger.debug("%s", line)
            self.__logger.debug("Kickstart file => END")

            self.__preParserFile(data)
            self.__parserFile(data)
        except PKVMError:
            raise
        except Exception:
            raise PKVMError("081")

    def onPreInstall(self, model):
        """
        Handles the pre install event

        Args:
            model (Model): installation data
        """
        try:

            self.__logger.info("Executing InstallAuto module (EVT_PRE_INSTALL)")
            # @TODO: use KernelCmdLine class
            # work on bootline command file.
            with open(CMD_FILE) as fd:
                # parser and store bootline command
                cmdLine = fd.read().rstrip().split()
            self.__logger.debug("Bootline:")
            for line in pprint.pformat(cmdLine).split('\n'):
                self.__logger.debug(line)

            # @TODO: use KernelCmdLine class
            # @TODO: move to another place
            # liveDVD parser
            for i in cmdLine:
                if i == "root=live:LABEL=POWERKVM_LIVECD":
                    self.__logger.debug(
                        "Boot from liveDVD, trying to start network...")
                    Network.startLiveDVDNetwork()
                    break

            # @TODO: use KernelCmdLine class
            # kickstart parser
            for i in cmdLine:
                if i.startswith('kvmp.inst.auto='):
                    self.__kickstartFile = i[i.find('=') + 1:]
                    self.__logger.debug("Kickstart file from bootline: %s",
                                        self.__kickstartFile)
                    self.__ksParser()
                    break

            self.__logger.debug("Data Structure:")
            for line in pprint.pformat(self.__model).split('\n'):
                self.__logger.debug(line)
            for key, value in self.__model.iteritems():
                model.insert(key, value)
        except PKVMError:
            raise
        except Exception:
            raise PKVMError("087")
