#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


"""This scripts provides the autolvm partitioning"""

#
# IMPORTS
#
import logging
import os
import shutil

from blivet.errors import LVMError
from blivet.size import Size
import blivet.devicelibs.lvm as lvm

from .partitioner import Partitioner
from controller.pkvmerror import PKVMError
from model.config import BOOT_SIZE
from model.config import CLEANUP_DIRS
from model.config import INSTALL_ROOT_DIR
from model.config import LVS_METADATA
from model.config import LV_PREFIX
from model.config import LV_SWAP
from model.config import LV_SYSTEM
from model.config import LV_VAR
from model.config import PKVM2_LVM_STRUCTURE
from model.config import PKVM2_LV_LOG
from model.config import PKVM2_LV_ROOT
from model.config import PKVM2_VG_DATA
from model.config import PKVM2_VG_LOG
from model.config import PKVM2_VG_ROOT
from model.config import PREP_SIZE
from model.config import REMAINING_SIZE_LV


#
# CONSTANTS
#
SYSTEMD_TIMEOUT = 300

MINIMAL_PVLOG_SIZE = Size('2GiB')


#
# CODE
#
class AutoPartitionerLVM(Partitioner):
    """
    Configures an automatic lvm partition layout.
    """

    def __init__(self):
        """
        Creates an autoLVM partition layout.
        """
        super(AutoPartitionerLVM, self).__init__()
        self.__logger = logging.getLogger(__name__)

    def setupAutoLVM(self, disks, lvsSizes):
        """
        Configures the partitioning layout desired and execute it.

        Args:
            disks ([str]): target disk devices names
            lvsSizes (dict): logical volumes sizes
        """
        layout = {}

        layout['partitions'], pvs = self.__partitionsLayout(disks)

        layout['vgs'] = self.__vgsLayout(pvs)
        layout['lvs'] = self.__lvsLayout(lvsSizes)

        # allocate the layout in disk
        self.setup(layout)

        self.__logger.info('Queue: %s', str(self.getPartitionLayout()))


    def __findPrepAndBoot(self, vgs):
        """
        Locate prep and boot partitions in an installation's volume
            groups

        Args:
            vgs ([blivet.devices.LVMVolumeGroupDevice]): volume groups

        Returns:
            (blivet.devices.PartitionDevice,
                blivet.devices.PartitionDevice): prep and boot
                    partitions
            (None, None): if multiple or no partitions found
        """
        storage = self._getStorage()
        partitions = storage._getBlivet().partitions

        installationPvs = []
        for vg in vgs:
           installationPvs += vg.pvs

        installationDisks = [pv.disk for pv in installationPvs]

        prepAndBoot = {'PReP':[],
                       'boot':[]}

        for partition in partitions:
            if self.isPrep(partition):
                if partition.disk in installationDisks:
                    prepAndBoot['PReP'].append(partition)
            elif self.isBoot(partition):
                if partition.disk in installationDisks:
                    prepAndBoot['boot'].append(partition)

        for partitionType, partitionList in prepAndBoot.iteritems():
            if len(partitionList) == 0:
                self.__logger.warn("No {} partition "
                                   "found".format(partitionType))
                return (None, None)
            elif len(partitionList) > 1:
                self.__logger.warn("Multiple {} partitions in the same "
                                   "installation.".format(partitionType))
                return (None, None)

        prep = prepAndBoot['PReP'][0]
        boot = prepAndBoot['boot'][0]

        if prep.disk != boot.disk:
            self.__logger.warn("PReP and boot are not on the same disk")
            # PKVM installs prep and boot on the same disk, failing
            # here to be on the safe side
            return (None,None)

        return (prep, boot)

    def getPreviousPkvmInstallations(self):
        """
        Find all PowerKVM instances installed in the system.

        Returns:
            dict: PowerKVM volume groups.
                Dict keys-values:
                    volume group name: PowerKVM volume groups information dict
                        Dict keys-values:
                            rootVg: blivet.devices.LVMVolumeGroupDevice
                            vgs: list of blivet.devices.LVMVolumeGroupDevice
                            version: int (2 or 3)
                            partitions: dict with the prep and boot partitions
                                Keys: prep, boot.
                                Values: corresponding
                                    blivet.devices.PartitionDevice.

        """
        storage = self._getStorage()

        lvmInfo = storage.lvmInfo()
        systemVgs = lvmInfo['vgs']

        pkvm2VgNames = PKVM2_LVM_STRUCTURE.keys()

        pkvmVgs = {}
        pkvm2Installation = {}

        for vg in systemVgs:
            version = self.getVolumeGroupPkvmVersion(vg)
            if version == '2':
                pkvm2Installation[vg.name] = vg
            elif version == '3+':
                prep, boot = self.__findPrepAndBoot([vg])
                if not prep or not boot:
                    self.__logger.warn("Found a PowerKVM 3 installation but "
                                       "can't find prep and/or boot "
                                       "partitions.")
                    continue
                pkvmVgs[vg.name] = {'rootVg': vg,
                                    'vgs': [vg],
                                    'version': 3,
                                    'partitions': {'prep': prep,
                                                   'boot': boot}}
            else:
                # user might have other volume groups, ignore them.
                continue

        # Make sure the PKVM 2 installation contains all 4 volume groups
        if set(pkvm2Installation.keys()) == set(pkvm2VgNames):
            vgs = pkvm2Installation.values()
            prep, boot = self.__findPrepAndBoot(vgs)
            if prep and boot:
                # We don't need to include all the installation's volume
                # groups, only the root volume group is needed. That is ok
                # as long as the disk partitioning structure contains the
                # logical volumes from the other volume groups.
                vgRoot = pkvm2Installation[LV_PREFIX + PKVM2_VG_ROOT]
                pkvmVgs[LV_PREFIX + PKVM2_VG_ROOT] = {'rootVg': vgRoot,
                                                      'vgs': vgs,
                                                      'version': 2,
                                                      'partitions': {'prep': prep,
                                                                     'boot': boot}}
            else:
                self.__logger.warn("Found a PowerKVM 2 installation, but "
                                   "can't find prep and/or boot partitions.")

        if not pkvmVgs:
            self.__logger.warn("No valid PowerKVM installation was found")

        return pkvmVgs


    def getVolumeGroupPkvmVersion(self, vg):
        """
        Check if the volume group belongs to a PKVM 2 or 3+
            installation

        Args:
            vg (blivet.devices.LVMVolumeGroupDevice): volume group to
                be checked

        Returns:
            str: PowerKVM version (2 or 3+) or None in case the volume
                group does not belong to a PowerKVM installation.
        """

        # Check all logical volumes from a volume group and make
        # sure their names are a superset of the names provided by
        # a PowerKVM installation. A superset is checked instead of
        # an exact match because user may have manually created
        # additional logical volumes after the installation.
        lvNamePrefix = vg.name + '-' + LV_PREFIX
        vgLvs = {lv.name: lv for lv in vg.lvs}

        pkvmLvNames = [lvNamePrefix + lvName for lvName in LVS_METADATA.keys()]

        if set(vgLvs.keys()).issuperset(set(pkvmLvNames)):
            return '3+'

        pkvm2VgNames = PKVM2_LVM_STRUCTURE.keys()

        # The name suffices to tell if a VG is from a PKVM 2 installation
        if vg.name in pkvm2VgNames:
            # We only support one PKVM 2 installation. If there are others,
            # its LVs will be inactive and will not be detected by blivet.
            if vg.lvs:
                return '2'
        return None


    def reuseCurrentAutoLVM(self, partitions):
        """
        Reuses the partition layout to reinstall the system.

        Args:
            partitions (dict):
                prep and boot partitions.
        """
        lvs = self.targetLVMInfo['lvs']
        for lv in lvs:
            lvNameWithIBMPrefix = lv.name.replace(lv.vg.name + '-', '')
            lvName = lvNameWithIBMPrefix.replace(LV_PREFIX, '')

            if lvName == PKVM2_LV_LOG:
                mountPoint = '/var/log'
            else:
                if lvName == PKVM2_LV_ROOT:
                    lvName = LV_SYSTEM
                mountPoint = LVS_METADATA[lvName]["mountPoint"]
            self.addMountPoint(mountPoint, lv)

            # Do not preserve the logical volume in which PowerKVM
            # is installed and the one used as swap.
            if lvName == LV_SYSTEM or lvName == LV_SWAP:
                self.formatPartition(lv, LVS_METADATA[lvName]['fsType'])

        self.addMountPoint('prep', partitions['prep'])
        self.formatPartition(partitions['prep'], 'prepboot')

        self.addMountPoint('/boot', partitions['boot'])
        self.formatPartition(partitions['boot'], 'ext4')

        self.__logger.info('Queue: %s', str(self.getPartitionLayout()))


    def updateFstab(self):
        """
        Creates a fstab file specific for this auto lvm setup and
            writes the file in chroot.
        """

        deviceSpecifiers = self.getMountPoints()

        fstab = ("%s\t/boot\text4\tdefaults,x-systemd.device-timeout=%d"
                 "\t1\t2\n") % (deviceSpecifiers["/boot"], SYSTEMD_TIMEOUT)

        for lvName, lvMetadata in LVS_METADATA.items():
            mountPoint = lvMetadata["mountPoint"]
            if mountPoint == "/var" and mountPoint not in deviceSpecifiers:
                mountPoint = "/var/log"
            deviceSpec = deviceSpecifiers[mountPoint]
            fstab += ("%s\t%s\t%s\tdefaults"
                      "\t%s\t%s\n") % (deviceSpec, mountPoint,
                                       lvMetadata["fsType"],
                                       lvMetadata["devBackup"],
                                       lvMetadata["fsCheckPriority"])
        super(AutoPartitionerLVM, self).updateFstab(fstab)


    def __vgsLayout(self, pvs):
        """
        Returns the VG layout for this automatic partitioning.

        Args:
            pvs ([?]): physical volumes

        Returns:
            [dict]: list of volume groups
        """
        return [
            {
                'pvs'  : pvs,
                'name' : self.systemVGName
            }
        ]


    def __lvsLayout(self, lvsSizes):
        """
        Returns the LV layout for this automatic partitioning.

        Args:
            lvsSizes (dict): logical volumes sizes

        Returns:
            [dict]: list of logical volumes
        """

        lvsLayout = []
        for lvName, lvMetadata in LVS_METADATA.items():
            lvLayout = {
                'vg'     : self.systemVGName,
                'name'   : LV_PREFIX + lvName,
                'mount'  : lvMetadata["mountPoint"],
                'fsType' : lvMetadata['fsType'],
                'grow'   : True
            }
            if lvName != REMAINING_SIZE_LV:
                lvLayout['size'] = str(lvsSizes[lvName]["size"]) + 'GiB'
            else:
                lvLayout['size'] = 'ALL_FREESPACE'
                remainingSizeLvLayout = lvLayout
            if lvName != REMAINING_SIZE_LV:
                lvsLayout.append(lvLayout)
        # blivet creates LVs in the order defined in lvsLayout list, so the
        # LV which will use disks remaining size must be at the end of the list
        lvsLayout.append(remainingSizeLvLayout)

        return lvsLayout

    def __partitionsLayout(self, disks):
        """
        Returns the basic parititon layout for the automatic
            partitioning.

        Args:
            disks ( list of str): disk devices names

        Returns:
            ([dict], [dict]): list of partitions in the single volume
            group
        """

        # generate list of Blivet disks from list of disks
        blivetSystemDisks = self.getSystemInformation()["disks"]
        blivetDisks = []
        for blivetSystemDisk in blivetSystemDisks:
            if blivetSystemDisk.name in disks:
                blivetDisks.append(blivetSystemDisk)

        partitions = [
                {
                    'disk'   : blivetDisks[0],
                    'name'   : 'prep',
                    'size'   : PREP_SIZE,
                    'fsType' : 'prepboot',
                    'boot'   : True,
                    'type'   : None,
                    'mount'  : 'prep',
                    'weight' : 1000000
                    },
                {
                    'disk'   : blivetDisks[0],
                    'name'   : 'boot',
                    'size'   : BOOT_SIZE,
                    'fsType' : 'ext4',
                    'boot'   : False,
                    'type'   : None,
                    'mount'  : '/boot',
                    'weight' : 800000
                    },
                ]

        pvs = []

        for blivetDisk in blivetDisks:
            diskName = 'root' + blivetDisk.name

            partitions += [
                    {
                        'disk'   : blivetDisk,
                        'name'   : diskName,
                        'size'   : 'ALL_FREESPACE',
                        'fsType' : 'lvmpv',
                        'boot'   : False,
                        'type'   : None,
                        'mount'  : None,
                        'weight' : 0
                        },
                    ]

            pvs.append(diskName)


        return (partitions,
                pvs)

    def cleanupVar(self):
        """
        Remove directories that could leave the system in a
            inconsistent state after a reinstallation.
        """

        for directory in CLEANUP_DIRS:
            if os.path.isdir(INSTALL_ROOT_DIR + directory):
                shutil.rmtree(INSTALL_ROOT_DIR + directory)

    def getAffectedDisks(self):
        preservedPKVM2VGs = [LV_PREFIX + PKVM2_VG_DATA,
                             LV_PREFIX + PKVM2_VG_LOG]
        disks = []
        for vg in self.targetLVMInfo['vgs']:
            if vg.name not in preservedPKVM2VGs:
                for pv in vg.pvs:
                    disks.append(pv.disk.name)
        # Remove duplicates in case it's a PKVM 2 LVM layout (all
        # VGs share the same disk) or if the user added PVs with
        # partitions of the same disk to two volume groups that
        # will not be preserved (the underlying disk would be the
        # same for the two VGs).
        return list(set(disks))
