#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from datetime import datetime

from snack import *

from .utils import genDateStr, genTimeStr
from model.config import LV_SYSTEM
from modules.i18n.i18n import getLangConf
from viewer.__data__ import BACK
from viewer.__data__ import BRIDGE
from viewer.__data__ import DATE_TIME
from viewer.__data__ import DEVICE
from viewer.__data__ import DHCP_IS_ENABLE
from viewer.__data__ import DNSSETUP_DNS_LIST_LABEL
from viewer.__data__ import DNSSETUP_SEARCH_LABEL
from viewer.__data__ import GATEWAY
from viewer.__data__ import HOSTNAME
from viewer.__data__ import IBM_POWERKVM
from viewer.__data__ import INSTALLATION_SUMMARY
from viewer.__data__ import IP_ADDRESS
from viewer.__data__ import NETMASK
from viewer.__data__ import NETWORK
from viewer.__data__ import NO
from viewer.__data__ import NTP_DISABLED
from viewer.__data__ import NTP_SERVERS
from viewer.__data__ import OK
from viewer.__data__ import PASSWORD_IS_SET
from viewer.__data__ import PRESERVE_LVS
from viewer.__data__ import SYSTEM_CLOCK_USES_UTC
from viewer.__data__ import TIMEZONE
from viewer.__data__ import YES

#
# CONSTANTS
#

#
# CODE
#
class InstallationSummary(object):
    """
    Builds a screen to show a confirmation message before wipe root
        device and reinstall system.
    """

    def __init__(self, screen, params):
        """
        Constructor.

        Args:
            screen (SnackScreen): SnackScreen instance

            params (dict): screen input parameters. Valid keys:
                disks list disks
                installData model.Model installation metadata
        """
        self.__screen = screen
        self.__model = params["installData"]
        self.__disks = self.__model.get("targetDisks")
        self.__preserveLVs = self.__model.get("action") == "reinstall"

        self.__textbox = Textbox(60, 10, "", scroll = 1)
        self.__msg = TextboxReflowed(60, INSTALLATION_SUMMARY.localize())

        self.__buttonsBar = ButtonBar(self.__screen, [(OK.localize(), "ok"),
            (BACK.localize(), "back")])

        self.__grid = GridForm(self.__screen, IBM_POWERKVM.localize() % params["productVersion"], 1, 3)
        self.__grid.add(self.__msg, 0, 0)
        self.__grid.add(self.__textbox, 0, 1)
        self.__grid.add(self.__buttonsBar, 0, 2)

    def run(self):
        """
        Draws the screen

        Returns:
            dict: screen output parameters. Valid keys:
            pressedButton str pressed button ID
        """

        networks = self.__model.get('network')
        dtime = datetime.now()

        dns_data = self.__model.get('dns')
        password = self.__model.get('pass')
        timezone = self.__model.get('tz')
        hostname = self.__model.get('hostname')
        utc = self.__model.get('isUTC')

        summary = '\n'
        if self.__preserveLVs:
            summary += PRESERVE_LVS.localize() + '\n'

        summary += DEVICE.localize() + '\n'

        summary += ', '.join(self.__disks)

        summary += '\n\n' + PASSWORD_IS_SET.localize()

        if hostname:
            summary += "\n\n" + HOSTNAME.localize() + hostname

        if networks:
            summary += "\n\n" + NETWORK.localize()

            for network in networks:
                summary += "\n" + DEVICE.localize() +  network['device']

                if network['bootProto'] == 'dhcp':
                    summary += "\n" + DHCP_IS_ENABLE.localize()
                else:
                    if network['ip']:
                        summary += "\n" + IP_ADDRESS.localize() + network['ip']

                    if network['gateway']:
                        summary += "\n" + GATEWAY.localize() + network['gateway']

                    if network['netmask']:
                        summary += "\n" + NETMASK.localize() + network['netmask']

                summary += "\n" + BRIDGE.localize()
                summary += YES.localize() if network['bridge'] else NO.localize()


        if dns_data:
            summary += "\n\nDNS\n"

            # Check if the user entered at least one domain name server on the
            # DNS configuration screen.
            if len(dns_data['dnsList']) > 0:
                dnsListLabel = DNSSETUP_DNS_LIST_LABEL.localize() + ': '
                padding = len(dnsListLabel)
                summary += dnsListLabel + dns_data['dnsList'][0] + '\n'

                for ns in dns_data['dnsList'][1:]:
                    summary += padding * ' ' + '%s\n' % ns

            if dns_data['search']:
                summary += DNSSETUP_SEARCH_LABEL.localize() + ": "
                summary += str(dns_data['search']) + "\n"


        summary += "\n" + TIMEZONE.localize() + timezone + "\n"

        summary += "\n" + SYSTEM_CLOCK_USES_UTC.localize()

        summary += YES.localize() if utc else NO.localize()

        tmp = [dtime.year, dtime.month, dtime.day, dtime.hour, dtime.minute,
               dtime.second]
        langConf = getLangConf()
        dtimeStr = map(str, tmp)

        dateString = genDateStr(langConf['datetime'],
                                dtimeStr[0], # y
                                dtimeStr[1], # m
                                dtimeStr[2]) # d

        summary += '\n\n{}{}'.format(DATE_TIME.localize(),
                                     dateString)

        summary += ' ' + genTimeStr(dtimeStr[3], # h
                                    dtimeStr[4], # m
                                    dtimeStr[5]) # s

        ntpservers = self.__model.get('ntpservers')
        summary += "\n\n"

        if ntpservers:
            summary += NTP_SERVERS.localize() + "\n"
            for i in self.__model.get('ntpservers'):
                summary += "* "+ i + "\n"
        else:
            summary += NTP_DISABLED.localize() + "\n"

        summary += "\n\n"

        self.__textbox.setText(summary)

        result = self.__grid.run()
        self.__screen.popWindow()

        return {"pressedButton": self.__buttonsBar.buttonPressed(result)}
