#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from snack import *

from viewer.__data__ import BACK
from viewer.__data__ import ENABLE_LABEL
from viewer.__data__ import ERROR_INVALID_ENTRY
from viewer.__data__ import ERROR_INVALID_ENTRY_MSG
from viewer.__data__ import NETWORKCFG_BRIDGE_LABEL
from viewer.__data__ import NETWORKCFG_DEVICE_CONFIG
from viewer.__data__ import NETWORKCFG_DEVICE_LABEL
from viewer.__data__ import NETWORKCFG_DHCP_LABEL
from viewer.__data__ import NETWORKCFG_GATEWAY_LABEL
from viewer.__data__ import NETWORKCFG_IP_LABEL
from viewer.__data__ import NETWORKCFG_MACADDR_LABEL
from viewer.__data__ import NETWORKCFG_NETMASK_LABEL
from viewer.__data__ import OK


#
# CONSTANTS
#


#
# CODE
#
class NetworkConfig:
    """
    Represents the network interface configuration screen
    """

    def __init__(self, screen, params):
        """
        Constructor

        Args:
            screen (SnackScreen): SnackScreen instance

            params (dict): screen input parameters. Valid keys:
                networkInterfaceName str network interface name
                networkInterfaceMacAddress str network interface MAC
                    address
        """
        self.__task = [False]
        self.__screen = screen
        self.__nwInterfaceName = params["networkInterfaceName"]
        self.__nwInterfaceMacAddress = params["networkInterfaceMacAddress"]
        self.__nwInterface = params["networkInterface"]
        self.__activeNwInterfaces = params["activeNwInterfaces"]

        self.__hwdev = Entry(20, "%s" % self.__nwInterfaceName)
        self.__hwdev.setFlags(FLAG_DISABLED, FLAGS_SET)

        self.__macAddress = Entry(20, self.__nwInterfaceMacAddress)
        self.__macAddress.setFlags(FLAG_DISABLED, FLAGS_SET)

        self.__dhcp = Checkbox("")
        self.__ip = Entry(20, "")
        self.__netmask = Entry(20, "")
        self.__gateway = Entry(20, "")
        self.__bridge = Checkbox("")
        self.__enable = Checkbox("")

        self.__hwdevLabel = Label(NETWORKCFG_DEVICE_LABEL.localize())
        self.__macAddressLabel = Label(NETWORKCFG_MACADDR_LABEL.localize())
        self.__dhcpLabel = Label(NETWORKCFG_DHCP_LABEL.localize())
        self.__ipLabel = Label(NETWORKCFG_IP_LABEL.localize())
        self.__netmaskLabel = Label(NETWORKCFG_NETMASK_LABEL.localize())
        self.__gatewayLabel = Label(NETWORKCFG_GATEWAY_LABEL.localize())
        self.__bridgeLabel = Label(NETWORKCFG_BRIDGE_LABEL.localize() % self.__nwInterfaceName)
        self.__enableLabel = Label(ENABLE_LABEL.localize())

        self.__contentGrid = Grid(2, 8)
        self.__contentGrid.setField(self.__hwdevLabel, 0, 0, anchorLeft=1)
        self.__contentGrid.setField(self.__macAddressLabel, 0, 1, anchorLeft=1)
        self.__contentGrid.setField(self.__dhcpLabel, 0, 2, anchorLeft=1)
        self.__contentGrid.setField(self.__ipLabel, 0, 3, anchorLeft=1)
        self.__contentGrid.setField(self.__netmaskLabel, 0, 4, anchorLeft=1)
        self.__contentGrid.setField(self.__gatewayLabel, 0, 5, anchorLeft=1)
        self.__contentGrid.setField(self.__bridgeLabel, 0, 6, anchorLeft=1)
        self.__contentGrid.setField(self.__enableLabel, 0, 7, anchorLeft=1)

        self.__contentGrid.setField(self.__hwdev, 1, 0, (1, 0, 0, 0))
        self.__contentGrid.setField(self.__macAddress, 1, 1, (1, 0, 0, 0))
        self.__contentGrid.setField(self.__dhcp, 1, 2, (1, 0, 0, 0), anchorLeft=1)
        self.__contentGrid.setField(self.__ip, 1, 3, (1, 0, 0, 0))
        self.__contentGrid.setField(self.__netmask, 1, 4, (1, 0, 0, 0))
        self.__contentGrid.setField(self.__gateway, 1, 5, (1, 0, 0, 0))
        self.__contentGrid.setField(self.__bridge, 1, 6, (1, 0, 0, 0), anchorLeft=1)
        self.__contentGrid.setField(self.__enable, 1, 7, (1, 0, 0, 0), anchorLeft=1)

        self.__dhcp.setCallback(self.useDynamicCheckBox)

        self.__buttonsBar = ButtonBar(self.__screen, [(OK.localize(), "ok"),
                                      (BACK.localize(), "back")])


        self.useDynamicCheckBox()

    def useDynamicCheckBox(self):
        """
        Handles the dhcp check box
        """
        if self.__dhcp.selected():
            state = FLAGS_SET
        else:
            state = FLAGS_RESET

        for i in self.__ip, self.__netmask, self.__gateway:
            i.setFlags(FLAG_DISABLED, state)

    def __hasLink(self):
        """
        Return the link status of the NIC

        Returns:
            bool: True if NIC has link
        """
        link = False
        if self.__nwInterfaceName in self.__activeNwInterfaces:
            link = True
        return link

    def run(self):
        """
        Draws the screen

        Returns:
            dict: screen output parameters. Valid keys:
            pressedButton str pressed button ID
            network dict network data
        """
        # dhcp
        if self.__nwInterface.getBootProto() == "dhcp":
            self.__dhcp.setValue('*')

        # bridge
        if self.__nwInterface.hasBridge():
            self.__bridge.setValue('*')

        # onboot
        if self.__hasLink():
            if self.__nwInterface.getOnBoot():
                self.__enable.setValue('*')
        else:
            self.__enable.setFlags(FLAG_DISABLED, FLAGS_SET)

        self.__ip.set(self.__nwInterface.getIp())
        self.__netmask.set(self.__nwInterface.getNetmask())
        self.__gateway.set(self.__nwInterface.getGateway())

        self.useDynamicCheckBox()


        # struct the data entered for returning
        network = {}

        validInput = False
        while not validInput:
            self.__grid = GridForm(self.__screen, NETWORKCFG_DEVICE_CONFIG.localize(), 1, 3)
            self.__grid.add(self.__contentGrid, 0, 0, (0, 0, 0, 1))
            self.__grid.add(self.__buttonsBar, 0, 1)
            self.__grid.setCurrent(self.__dhcp)
            result = self.__grid.run()
            self.__screen.popWindow()
            rc = self.__buttonsBar.buttonPressed(result)
            if rc == "back":
                return {"pressedButton": rc, "network": None}

            # @IMPROVEMENT: data validation should be made by model layer
            if self.__dhcp.selected() != (self.__ip.value() == "" and self.__gateway.value() == "" and self.__netmask.value() == "") and self.__enable.selected():
                self.showErrorEntry()
                continue

            validInput = True

        network['name'] = self.__nwInterfaceName
        network['mac'] = self.__nwInterfaceMacAddress
        network['dhcp'] = self.__dhcp.selected()
        network['ip'] = self.__ip.value()
        network['gateway'] = self.__gateway.value()
        network['netmask'] = self.__netmask.value()
        network['bridge'] = self.__bridge.selected()
        network['onboot'] = self.__enable.selected() and self.__hasLink()

        return {"pressedButton": rc,
                "network": network}

    def showErrorEntry(self):
        """
        Displays an error about the invalid entry
        """
        ButtonChoiceWindow(self.__screen, ERROR_INVALID_ENTRY.localize(),
                           ERROR_INVALID_ENTRY_MSG.localize(),
                           buttons=[(OK.localize(), 'ok')],
                           width=50)
