#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import socket

from blivet.size import Size
from snack import ButtonBar
from snack import ButtonChoiceWindow
from snack import Entry
from snack import FLAGS_SET
from snack import FLAG_DISABLED
from snack import Grid
from snack import GridForm
from snack import Label

from model.config import LV_DATA
from model.config import LV_SWAP
from model.config import LV_SWAP_MIN_SIZE
from model.config import LV_SYSTEM
from model.config import LV_SYSTEM_MIN_SIZE
from model.config import LV_VAR
from model.config import LV_VAR_MIN_SIZE
from model.config import LVS_METADATA
from model.config import LVS_UI_ORDER
from model.config import REMAINING_SIZE_LV
from viewer.__data__ import BACK
from viewer.__data__ import ERROR_SIZE_INVALID_VALUE
from viewer.__data__ import ERROR_SIZE_INVALID_VALUE_MSG
from viewer.__data__ import ERROR_SIZE_MIN_VALUE
from viewer.__data__ import ERROR_SIZE_MIN_VALUE_MSG
from viewer.__data__ import OK
from viewer.__data__ import RESIZE_GB
from viewer.__data__ import RESIZE_LOGICAL_VOLUME
from viewer.__data__ import SIZE_OF
from viewer.__data__ import SIZE_OF_MIN_VALUE
from viewer.__data__ import VOLUME_GROUP_NAME

#
# CONSTANTS
#
DEFAULT_VG_NAME='ibmpkvm_vg_root'


#
# CODE
#
class ResizeLogicalVolume:
    """
    Represents the screen to resize the logical volumes
    """

    def __init__(self, screen, params):
        """
        Constructor

        Args:
            screen (SnackScreen): SnackScreen instance
            totalSize (blivet Size object): Total size of all selected
                disk devices

        """

        self.__screen = screen

        self.__contentGrid = Grid(5, 5)

        self.totalSize = params["targetDisksTotalSize"]

        def addEntry(resizeLabel, line, value=""):
            """
            Add entry field

            Args:
                resizeLabel (string): Label name
                line (int): number of the line
                value (string): standard value
            """

            self.__volumeSizeLabel = Label(resizeLabel)
            self.__gbLabel = Label(RESIZE_GB.localize())

            self.__entry = Entry(7, value)

            self.__contentGrid.setField(self.__volumeSizeLabel, 0, line, anchorLeft=1)
            self.__contentGrid.setField(self.__entry, 1, line, (1, 0, 0, 0))
            self.__contentGrid.setField(self.__gbLabel, 2, line, anchorRight=1)

            return self.__entry

        line = 0
        self.__contentGrid.setField(Label(VOLUME_GROUP_NAME.localize()), 0, line, anchorLeft=1)
        self.__vgNameEntry = Entry(35, DEFAULT_VG_NAME)
        self.__vgNameEntry.setCallback(self.validateVgName)
        self.__contentGrid.setField(self.__vgNameEntry, 1, line, anchorLeft=1)
        line += 1

        lvsLabels = {}
        for lvName, lvDict in LVS_METADATA.items():
            labelData = {'mountPoint': lvDict["mountPoint"],
                         'minSize': lvDict["minSize"],
                         'sizeUnit': RESIZE_GB.localize()}
            lvsLabels[lvName] = SIZE_OF_MIN_VALUE.localize() % labelData

        labelData = {'mountPoint': LVS_METADATA[LV_DATA]["mountPoint"]}
        lvsLabels[LV_DATA] = SIZE_OF.localize() % labelData

        self.lvsEntries = {}
        for lvName in LVS_UI_ORDER:
            lvMetadata = LVS_METADATA[lvName]
            lvEntry = addEntry(lvsLabels[lvName], line, lvMetadata["minSize"])
            if lvName != REMAINING_SIZE_LV:
                lvEntry.setCallback(self.updateRemainingLvSizeLabel)
            else:
                lvEntry.setFlags(FLAG_DISABLED, FLAGS_SET)
            self.lvsEntries[lvName] = lvEntry
            line += 1
        remainingLvSize = self.calcRemainingLvSize()
        self.lvsEntries[REMAINING_SIZE_LV].set(str(remainingLvSize))
    # __init__()

    def calcRemainingLvSize(self):

        # remaining LV size is equal to (total - sum(other LVs sizes)
        remainingLvSize = self.totalSize
        for lvName, lvMetadata in LVS_METADATA.items():
            if lvName != REMAINING_SIZE_LV:
                remainingLvSize -= Size(self.lvsEntries[lvName].value() + 'GiB')
        return remainingLvSize.convertTo('GiB')

    def updateRemainingLvSizeLabel(self):
        """
        Update the label of the remaining logical volume size
        """

        # validate LVs sizes entries labels
        for lvName, lvMetadata in LVS_METADATA.items():
            if lvName == REMAINING_SIZE_LV:
                continue

            try:
                lvSize = Size(self.lvsEntries[lvName].value() + 'GiB')
            except ValueError:
                self.showErrorValueError()
                self.lvsEntries[lvName].set(lvMetadata["minSize"])
                continue

            if lvSize < Size(str(lvMetadata["minSize"]) + 'GiB'):
                self.showErrorMinSize(lvMetadata["mountPoint"], lvMetadata["minSize"])
                self.lvsEntries[lvName].set(lvMetadata["minSize"])

        # update remaining LV size entry label
        remainingLvSize = self.calcRemainingLvSize()
        self.lvsEntries[REMAINING_SIZE_LV].set(str(remainingLvSize))

    def validateVgName(self):
        """
        Make sure the volume group name field is not empty and remove
        leading and trailing whitespace.
        """

        strippedVgName = self.__vgNameEntry.value().strip()
        if strippedVgName == "":
            self.__vgNameEntry.set(DEFAULT_VG_NAME)
        else:
            self.__vgNameEntry.set(strippedVgName)


    def __runUI(self):
        """
        Shows the UI components and waits until the user presses a
            button
        """
        # Newt couples creation and display of UI components, so GridForm and
        # ButtonBar creation must be done here
        self.__buttonsBar = ButtonBar(self.__screen, [(OK.localize(),"ok"),
                                                      (BACK.localize(), "back")])
        self.__grid = GridForm(self.__screen, RESIZE_LOGICAL_VOLUME.localize(), 1, 2)
        self.__grid.add(self.__contentGrid, 0, 0, (0, 0, 0, 1))
        self.__grid.add(self.__buttonsBar, 0, 1)


        self.__screen.refresh()

        result = self.__grid.run()
        self.__screen.popWindow()

        return self.__buttonsBar.buttonPressed(result)


    def run(self):
        """
        Draws the screen

        Returns:
            int: status of operation
        """
        validInput = False
        vgName = ""
        lvsSizes = {}
        while not validInput:
            pressedButton = self.__runUI()

            if pressedButton == 'back':
                return {"pressedButton": pressedButton,
                        "vgName": vgName,
                        "lvsSizes": lvsSizes}

            remainingLvSize = Size(self.lvsEntries[REMAINING_SIZE_LV].value() + 'GiB')
            remainingLvMinimumSize = Size(LVS_METADATA[REMAINING_SIZE_LV]["minSize"] + 'GiB')

            if remainingLvSize < remainingLvMinimumSize:
                self.showErrorMinSize(LVS_METADATA[REMAINING_SIZE_LV]["mountPoint"], LVS_METADATA[REMAINING_SIZE_LV]["minSize"])
            else:
                validInput = True

        for lvName in LVS_METADATA.keys():
            if lvName == REMAINING_SIZE_LV:
                continue

            lvsSizes[lvName] = {}
            lvsSizes[lvName]['size'] = float(self.lvsEntries[lvName].value())

        vgName = self.__vgNameEntry.value()

        return {"pressedButton": pressedButton,
                "vgName": vgName,
                "lvsSizes": lvsSizes}

    def showErrorValueError(self):
        """
        Displays an error when the value is invalid
        """
        ButtonChoiceWindow(self.__screen, ERROR_SIZE_INVALID_VALUE.localize(),
                           ERROR_SIZE_INVALID_VALUE_MSG.localize(),
                           buttons=[(OK.localize(), 'ok')],
                           width=50)

    def showErrorMinSize(self, logicalVolume, mountPointSize):
        """
        Displays an error when the value is less than minimum
        """
        ButtonChoiceWindow(self.__screen, ERROR_SIZE_MIN_VALUE.localize(),
                           ERROR_SIZE_MIN_VALUE_MSG.localize() % {'mnt': logicalVolume, 'size': mountPointSize},
                           buttons=[(OK.localize(), 'ok')],
                           width=50)
