#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from snack import *

# @IMPROVEMENT: remove Blivet dependency from this screen
from blivet.devices import MultipathDevice

from model.config import MIN_DISK_SIZE
from viewer.__data__ import BACK
from viewer.__data__ import ERROR_DISK_SIZE
from viewer.__data__ import ERROR_DISKS_SIZES_MSG
from viewer.__data__ import ERROR_NO_DISK_FOUND
from viewer.__data__ import ERROR_NO_DISK_FOUND_MSG
from viewer.__data__ import ERROR_NO_DISK_SELECTED
from viewer.__data__ import ERROR_NO_DISK_SELECTED_MSG
from viewer.__data__ import HELP_LINE
from viewer.__data__ import IBM_POWERKVM
from viewer.__data__ import OK
from viewer.__data__ import SELECT_DEVICE_TO_INSTALL
from viewer.__data__ import SELECT_FILESYSTEM
from viewer.__data__ import SELECT_HELP_LINE
from viewer.__data__ import SELECT_ID
from viewer.__data__ import SELECT_MULTIPATH
from viewer.__data__ import SELECT_NAME
from viewer.__data__ import SELECT_SIZE
from viewer.__data__ import SELECT_TITLE
from viewer.__data__ import SELECT_VGRAID



#
# CONSTANTS
#

#
# CODE
#
class SelectHardDisk(object):
    """
    Show all hard disks found in the system
    """

    def __init__(self, screen, params):
        """
        Constructor

        Args:
            screen (SnackScreen): SnackScreen instance
        """
        self.__disks = params["systemDisks"]
        self.__productVersion = params["productVersion"]
        self.__screen = screen
        self.__msg = TextboxReflowed(40, SELECT_DEVICE_TO_INSTALL.localize())
        self.__checkBoxTree = CheckboxTree(5)
        self.__buttonsBar = ButtonBar(self.__screen, [(OK.localize(), "ok"),
            (BACK.localize(), "back")])


    def __runUI(self):

        # show UI
        # tricky: in Newt, form creation and form display are coupled in GridForm(), so
        # form creation must be done here
        self.__grid = GridForm(self.__screen, IBM_POWERKVM.localize() % self.__productVersion, 1, 3)
        self.__grid.add(self.__msg, 0, 0)
        self.__grid.add(self.__checkBoxTree, 0, 1, (0, 1, 0, 0))
        self.__grid.add(self.__buttonsBar, 0, 2)
        self.__grid.addHotKey('F3')
        self.__screen.pushHelpLine(SELECT_HELP_LINE.localize())

        # add each disk in the selection list
        for disk in self.__disks:
            self.__checkBoxTree.append('%s - %s - %s' % (disk["name"], disk["serial"], disk["size"]),
                               disk["name"])
        for disk in self.__selectedDisks:
            self.__checkBoxTree.setEntryValue(disk["name"], selected = 1)

        # get user input
        result = self.__grid.run()

        # handle hotkey to display detailed disk information to
        # the user
        selectedDisks = self.__checkBoxTree.getSelection()
        currentDisk = self.__checkBoxTree.getCurrent()
        if result == 'F3':
            action = "details"
        else:
            action = self.__buttonsBar.buttonPressed(result)

        self.__screen.popWindow()
        self.__screen.pushHelpLine(HELP_LINE.localize())
        return action, selectedDisks, currentDisk

    def run(self, selectedDisks):

        """
        Draws the screen

        Args:
            disks ([dict]): list of dicts. Valid dict keys: name,
                serial, size

        Returns:
            int: sucess status
        """

        self.__selectedDisks = selectedDisks
        action, self.__selectedDisks, currentDisk = self.__runUI()
        if action == "details":
            return {"action": "details", "diskName": currentDisk}
        if action == "back":
            return {"action": "back"}
        if action == "ok":
            return {"action": "ok", "selectedDisks": self.__selectedDisks}


    def showDiskInformation(self, disk):
        """
        Displays detailed information about the selected disk

        Args:
            disk (dict): disk metadata
        """

        # data format
        if "mpathSlaves" not in disk:
            mpathSlavesStr = 'No'
        else:
            mpathSlavesStr = ', '.join(disk["mpathSlaves"])
        table = []
        table.append('{:15s} {:55s}'.format(SELECT_ID.localize(), disk["serial"]))
        table.append('{:15s} {:35s}'.format(SELECT_SIZE.localize() + ':', disk["size"]))
        table.append('{:15s} {:55s}\n'.format(SELECT_MULTIPATH.localize(), mpathSlavesStr))
        table.append('{:10s} {:10s} {:30s} {:20s}'.format(SELECT_NAME.localize(), SELECT_SIZE.localize(), SELECT_FILESYSTEM.localize(), SELECT_VGRAID.localize()))

        for partition in disk["partitions"]:

            table.append('{:10s} {:10s} {:30s} {:20s}'.format(
                partition["name"],
                partition["size"],
                partition["format"],
                partition["extraInfo"]
                )
            )

        content = '\n'.join(table)


        ButtonChoiceWindow(self.__screen, "%s - %s" % (SELECT_TITLE.localize(), disk["name"]),
                           content,
                           buttons=[(OK.localize(), 'ok')],
                           width=70)


    def showErrorDiskSize(self):
        """
        Displays an error about the disk size
        """
        errorMsgData = {"minDiskSize": MIN_DISK_SIZE}
        ButtonChoiceWindow(self.__screen, ERROR_DISK_SIZE.localize(),
                           ERROR_DISKS_SIZES_MSG.localize() % errorMsgData,
                           buttons=[(OK.localize(), 'ok')],
                           width=50)


    def showErrorNoDisk(self):
        """
        Displays an error when no disk found
        """
        ButtonChoiceWindow(self.__screen, ERROR_NO_DISK_FOUND.localize(),
                           ERROR_NO_DISK_FOUND_MSG.localize(),
                           buttons=[(OK.localize(), 'ok')],
                           width=50)

    def showErrorNoDiskSelected(self):
        """
        Displays an error when no disk is selected on disk selection
            screen.
        """
        ButtonChoiceWindow(self.__screen, ERROR_NO_DISK_SELECTED.localize(),
                           ERROR_NO_DISK_SELECTED_MSG.localize(),
                           buttons=[(OK.localize(), 'ok')],
                           width=50)
