#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from snack import ButtonBar, Grid, GridForm, Listbox, SnackScreen, Textbox

from model.config import CLEANUP_DIRS
from model.config import PRESERVABLE_MOUNT_POINTS
from viewer.__data__ import BACK
from viewer.__data__ import DISK_LAYOUT
from viewer.__data__ import EXCEPT
from viewer.__data__ import HELP_LINE
from viewer.__data__ import IBM_POWERKVM
from viewer.__data__ import LVS_TO_PRESERVE
from viewer.__data__ import MOUNT_POINTS_TO_PRESERVE
from viewer.__data__ import OK
from viewer.__data__ import SELECT_INSTALLATION


LEFT_TOP=[0, 0]
RIGHT_TOP=[1, 0]
LEFT_BOTTOM=[0, 1]
TEXT_BOX_HEIGHT=10
OK_LABEL = 'ok'
BACK_LABEL = 'back'


class SelectPreviousInstallation:
    """
    Screen that allows the selection of a previous PowerKVM
        installation.
    """

    def __init__(self, screen, params):
        """
        Constructor

        Args:
            screen (SnackScreen): SnackScreen instance
            params (dict): input parameters
        """

        self.__vgs = params['previousPkvmInstallations']
        self.__screen = screen
        self.__screen.pushHelpLine(HELP_LINE.localize())

        self.__gridForm = GridForm(self.__screen,
                                   SELECT_INSTALLATION.localize(), 1, 2)
        self.__upperGrid = Grid(2,1)

        # The scroll bar will be shown if the number of installations
        # to display is greater than the size of the Listbox. The - 2
        # is to account for the version labels.
        self.__listBox =Listbox(width=34, height=TEXT_BOX_HEIGHT,
                                scroll=len(self.__vgs) > TEXT_BOX_HEIGHT - 2,
                                returnExit=False)
        self.__listBox.setCallback(self.__updateTextBox)
        self.__textBox = Textbox(width=36, height=TEXT_BOX_HEIGHT, text="",
                                 wrap=True)
        self.__buttonBar = ButtonBar(self.__screen,
                                     buttonlist=[(OK.localize(), OK_LABEL),
                                                (BACK.localize() , BACK_LABEL)])

        self.__upperGrid.setField(self.__listBox, *LEFT_TOP, anchorLeft=True)
        self.__upperGrid.setField(self.__textBox, *RIGHT_TOP,
                                  anchorRight=True, padding=(1,0,-1,0))

        self.__gridForm.add(self.__upperGrid, *LEFT_TOP)
        self.__gridForm.add(self.__buttonBar, *LEFT_BOTTOM, padding=(0,1,0,0))

        self.__initListBox(self.__vgs)

    def __initListBox(self, powerKvmVgs):
        """
        Set volume groups list items.


        Args:
            powerKvmVgs (dict): volume group name - volume group information
                (dict)
        """

        for vgInfo in powerKvmVgs.values():
            if vgInfo['version'] == 3:
                self.__listBox.append(IBM_POWERKVM.localize() % "3.1.0 " +
                                      DISK_LAYOUT.localize() + ":",
                                      "labelpkvm3")
                break

        pkvm2 = ""
        for vgName, vgInfo in powerKvmVgs.items():
            if vgInfo['version'] == 2:
                pkvm2 = vgName
                continue
            self.__listBox.append(" " + vgName, vgName)

        if pkvm2:
            self.__listBox.append(IBM_POWERKVM.localize() % "2.1.x " +
                                  DISK_LAYOUT.localize() + ":", "labelpkvm2")
            self.__listBox.append(" " + pkvm2, pkvm2)

    def __updateTextBox(self):
        """
        Display information about the selected volume group in the text
            box.
        """

        if self.__currItemIsPkvmLabel():
            # The Textbox does not clear itself before a setText, so we
            # need to do it by hand.
            self.__textBox.setText("\n"*TEXT_BOX_HEIGHT)
            return

        selectedVGName = self.__listBox.current()
        version = self.__vgs[selectedVGName]['version']

        mountPoints = PRESERVABLE_MOUNT_POINTS["pkvm" + str(version)]

        # Move /var to end of list for aesthetic purposes
        if '/var' in mountPoints:
            mountPoints.remove('/var')
            mountPoints.append('/var')

        msg = MOUNT_POINTS_TO_PRESERVE.localize() + "\n\n"
        msg += '\n'.join(mountPoints)
        if version == 3:
            msg += ", " + EXCEPT.localize()
            msg += "\n  "
            msg += '\n  '.join(CLEANUP_DIRS)

        self.__textBox.setText("\n"*TEXT_BOX_HEIGHT)
        self.__textBox.setText(msg)

    def __currItemIsPkvmLabel(self):
        """
        Returns:
            bool: whether the currently selected item is a label
                indicating the PowerKVM version.
        """

        return self.__listBox.current()[:-1] == "labelpkvm"

    def run(self):
        """
        Draws the screen

        Returns:
            dict: button pressed, volume group selected
        """

        validInput = False
        while not validInput:
            result = self.__gridForm.run()

            pressedButton = self.__buttonBar.buttonPressed(result)
            if pressedButton == "back":
                break
            validInput = not self.__currItemIsPkvmLabel()

        self.__screen.popWindow()

        return {"pressedButton": pressedButton,
                "selectedVGName": self.__listBox.current()}
